/*	Public domain	*/

#include <agar/gui/begin.h>

/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_Surface *_Nullable AG_ReadSurface(AG_DataSource *_Nonnull);
extern DECLSPEC void AG_WriteSurface(AG_DataSource *_Nonnull, AG_Surface *_Nonnull);
extern DECLSPEC int AG_ReadPalette(AG_Palette *_Nonnull, AG_DataSource *_Nonnull);
extern DECLSPEC void AG_WritePalette(AG_DataSource *_Nonnull, const AG_Palette *_Nonnull);
__END_DECLS
/* Close generated block */

#include <agar/gui/close.h>
