#include <agar/gui/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticFont agFontAlgue;
extern DECLSPEC AG_StaticFont agFontAlgue_Bold;
extern DECLSPEC AG_StaticFont agFontAlgue_Italic;
extern DECLSPEC AG_StaticFont agFontAlgue_BoldItalic;
__END_DECLS
/* Close generated block */
#include <agar/gui/close.h>
